//Arduino 1.0+ Only
//Arduino 1.0+ Only

/***********************************************************************
Name: Altitude logger
Author: Peter Čamaj
Last modified: 28.01.2013
Version: 1.0
***********************************************************************/


/*BMP085 lybrary. based on code by  Jim Lindblom

 Get pressure, altitude, and temperature from the BMP085.
 Serial.print it out at 9600 baud to serial monitor.
 */

#include <Wire.h>

#define BMP085_ADDRESS 0x77  // I2C address of BMP085
#define DEVICE 0x50 //this is the device ID from the datasheet of the 24LC256

// set pin numbers:
const int buttonPin = 8;     // the number of the pushbutton pin
const int ledPin =  13;      // the number of the LED pin

// variables will change:
int buttonState = 0;         // variable for reading the pushbutton status


const unsigned char OSS = 1;  // Oversampling Setting

unsigned int tt = 0;


// Calibration values
int ac1;
int ac2;
int ac3;
unsigned int ac4;
unsigned int ac5;
unsigned int ac6;
int b1;
int b2;
int mb;
int mc;
int md;
byte * tmp;
float data2;
int  address;
float StartAltitude;
float temperature;
float altitude;
unsigned int addresss;
unsigned int adr;
int x = 0;
int row = 0;

// MCU calculate average value of altitude from NumRead measurements 
int NumRead= 50;
   
   

// b5 is calculated in bmp085GetTemperature(...), this variable is also used in bmp085GetPressure(...)
// so ...Temperature(...) must be called before ...Pressure(...).
long b5; 

void setup(){
  address=0;
  Serial.begin(9600);
  Wire.begin();
  // initialize the LED pin as an output:
  pinMode(ledPin, OUTPUT);      
  // initialize the pushbutton pin as an input:
  pinMode(buttonPin, INPUT); 

  bmp085Calibration();
  StartAltitude = averageAltitude(); // altitude when device start 
  
  // clear data in Excel
  Serial.println("CLEARDATA");
  //set table variables
  Serial.println("LABEL,Time,time(s),Altitude(m)");
  
}

void loop()
{ 
  digitalWrite(ledPin, HIGH); //LED ON
  buttonState = digitalRead(buttonPin); //read PIN
  
  if (buttonState == LOW) { 
    delay(3000);
    digitalWrite(ledPin, LOW);
    buttonState = digitalRead(buttonPin);
    
    if (buttonState == LOW){
      delay(1000);
      buttonState = digitalRead(buttonPin);
      while (buttonState == HIGH){
        digitalWrite(ledPin, HIGH);
        buttonState = digitalRead(buttonPin);
        //send data to PC
        float z = i2c_eeprom_read_float();
        row++;
        x++;
        digitalWrite(ledPin, LOW);
        if(z==9999.99){
          buttonState = LOW;
          while (1){}
        }
        else{
          //sending data to PC
          Serial.print("DATA,TIME,"); Serial.print(x); Serial.print(","); Serial.println(z);
          

        }
        
      }
    }
    else{
      
      
       while (1) { 
        digitalWrite(ledPin, LOW);
        delay(170); //callculed for Fvz=1Hz   
        digitalWrite(ledPin, HIGH);
        altitude = averageAltitude();
        altitude= altitude - StartAltitude;
        i2c_eeprom_write_float(altitude);
      
        Serial.println();
        Serial.print("Altitude: ");
        Serial.print(altitude, 2); //display 2 decimal places
        Serial.println(" M");
        tt++;
        Serial.println(tt);//line break\

        digitalWrite(ledPin, LOW);
            
        buttonState = digitalRead(buttonPin);
        if (buttonState == LOW)
        {
          i2c_eeprom_write_float(9999.99); //Stop FLOAT          
          digitalWrite(ledPin, HIGH);
          while (1){
          }
        }
      
    }
    }
  }
    else {
    digitalWrite(ledPin, HIGH); 
  }

}


void i2c_eeprom_write_float(float data){

  tmp = (byte *) &data;
  i2c_eeprom_write_byte(DEVICE, addresss,tmp[0]);
    delay(6);
  addresss++;
  i2c_eeprom_write_byte(DEVICE, addresss,tmp[1]);
    delay(6);
  addresss++;
  i2c_eeprom_write_byte(DEVICE, addresss,tmp[2]);
    delay(6);
   addresss++;
  i2c_eeprom_write_byte(DEVICE, addresss,tmp[3]);
  addresss++;
  delay(6);
 
 
}

float i2c_eeprom_read_float(){
  tmp = (byte *)&data2;
  tmp[0] =i2c_eeprom_read_byte(DEVICE, adr );
  adr++;
  tmp[1] =i2c_eeprom_read_byte(DEVICE, adr );
  adr++;
  tmp[2] =i2c_eeprom_read_byte(DEVICE, adr );
  adr++;
  tmp[3] =i2c_eeprom_read_byte(DEVICE, adr );  
  adr++;
  return data2;

}

void i2c_eeprom_write_byte( int deviceaddress, unsigned int eeaddress, byte data ) {
    int rdata = data;
    
    Wire.beginTransmission(deviceaddress);
    Wire.write((int)(eeaddress >> 8)); // MSB
    Wire.write((int)(eeaddress & 0xFF)); // LSB
    Wire.write(rdata);
    Wire.endTransmission();
  }
  


  byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress ) {
    byte rdata = 0xFF;
    Wire.beginTransmission(deviceaddress);
    Wire.write((int)(eeaddress >> 8)); // MSB
    Wire.write((int)(eeaddress & 0xFF)); // LSB
    Wire.endTransmission();
    Wire.requestFrom(deviceaddress,1);
    if (Wire.available()) rdata = Wire.read();
    return rdata;
  }




// Stores all of the bmp085's calibration values into global variables
// Calibration values are required to calculate temp and pressure
// This function should be called at the beginning of the program

float averageAltitude(){
  int i = 0;
  float at2 = 0;
  float at3 = 0;
  float at1=0;
  float temperature=0;
  float pressure=0;
  for(i; i < NumRead ; i++){
  temperature = bmp085GetTemperature(bmp085ReadUT()); //MUST be called first
  pressure = bmp085GetPressure(bmp085ReadUP());
  at1= calcAltitude(pressure); //Uncompensated caculation - in Meters 
  at2 = at1 + at2; 
  }
  at3 = at2 / NumRead;
  return at3;
}

void bmp085Calibration()
{
  ac1 = bmp085ReadInt(0xAA);
  ac2 = bmp085ReadInt(0xAC);
  ac3 = bmp085ReadInt(0xAE);
  ac4 = bmp085ReadInt(0xB0);
  ac5 = bmp085ReadInt(0xB2);
  ac6 = bmp085ReadInt(0xB4);
  b1 = bmp085ReadInt(0xB6);
  b2 = bmp085ReadInt(0xB8);
  mb = bmp085ReadInt(0xBA);
  mc = bmp085ReadInt(0xBC);
  md = bmp085ReadInt(0xBE);
}

// Calculate temperature in deg C
float bmp085GetTemperature(unsigned int ut){
  long x1, x2;

  x1 = (((long)ut - (long)ac6)*(long)ac5) >> 15;
  x2 = ((long)mc << 11)/(x1 + md);
  b5 = x1 + x2;

  float temp = ((b5 + 8)>>4);
  temp = temp /10;

  return temp;
}

// Calculate pressure given up
// calibration values must be known
// b5 is also required so bmp085GetTemperature(...) must be called first.
// Value returned will be pressure in units of Pa.
long bmp085GetPressure(unsigned long up){
  long x1, x2, x3, b3, b6, p;
  unsigned long b4, b7;

  b6 = b5 - 4000;
  // Calculate B3
  x1 = (b2 * (b6 * b6)>>12)>>11;
  x2 = (ac2 * b6)>>11;
  x3 = x1 + x2;
  b3 = (((((long)ac1)*4 + x3)<<OSS) + 2)>>2;

  // Calculate B4
  x1 = (ac3 * b6)>>13;
  x2 = (b1 * ((b6 * b6)>>12))>>16;
  x3 = ((x1 + x2) + 2)>>2;
  b4 = (ac4 * (unsigned long)(x3 + 32768))>>15;

  b7 = ((unsigned long)(up - b3) * (50000>>OSS));
  if (b7 < 0x80000000)
    p = (b7<<1)/b4;
  else
    p = (b7/b4)<<1;

  x1 = (p>>8) * (p>>8);
  x1 = (x1 * 3038)>>16;
  x2 = (-7357 * p)>>16;
  p += (x1 + x2 + 3791)>>4;

  long temp = p;
  return temp;
}

// Read 1 byte from the BMP085 at 'address'
char bmp085Read(unsigned char address)
{
  unsigned char data;

  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();

  Wire.requestFrom(BMP085_ADDRESS, 1);
  while(!Wire.available())
    ;

  return Wire.read();
}

// Read 2 bytes from the BMP085
// First byte will be from 'address'
// Second byte will be from 'address'+1
int bmp085ReadInt(unsigned char address)
{
  unsigned char msb, lsb;

  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();

  Wire.requestFrom(BMP085_ADDRESS, 2);
  while(Wire.available()<2)
    ;
  msb = Wire.read();
  lsb = Wire.read();

  return (int) msb<<8 | lsb;
}

// Read the uncompensated temperature value
unsigned int bmp085ReadUT(){
  unsigned int ut;

  // Write 0x2E into Register 0xF4
  // This requests a temperature reading
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF4);
  Wire.write(0x2E);
  Wire.endTransmission();

  // Wait at least 4.5ms
  delay(5);

  // Read two bytes from registers 0xF6 and 0xF7
  ut = bmp085ReadInt(0xF6);
  return ut;
}

// Read the uncompensated pressure value
unsigned long bmp085ReadUP(){

  unsigned char msb, lsb, xlsb;
  unsigned long up = 0;

  // Write 0x34+(OSS<<6) into register 0xF4
  // Request a pressure reading w/ oversampling setting
  Wire.beginTransmission(BMP085_ADDRESS);
  Wire.write(0xF4);
  Wire.write(0x34 + (OSS<<6));
  Wire.endTransmission();

  // Wait for conversion, delay time dependent on OSS
  delay(2 + (3<<OSS));

  // Read register 0xF6 (MSB), 0xF7 (LSB), and 0xF8 (XLSB)
  msb = bmp085Read(0xF6);
  lsb = bmp085Read(0xF7);
  xlsb = bmp085Read(0xF8);

  up = (((unsigned long) msb << 16) | ((unsigned long) lsb << 8) | (unsigned long) xlsb) >> (8-OSS);

  return up;
}

void writeRegister(int deviceAddress, byte address, byte val) {
  Wire.beginTransmission(deviceAddress); // start transmission to device 
  Wire.write(address);       // send register address
  Wire.write(val);         // send value to write
  Wire.endTransmission();     // end transmission
}

int readRegister(int deviceAddress, byte address){

  int v;
  Wire.beginTransmission(deviceAddress);
  Wire.write(address); // register to read
  Wire.endTransmission();

  Wire.requestFrom(deviceAddress, 1); // read a byte

  while(!Wire.available()) {
    // waiting
  }

  v = Wire.read();
  return v;
}

float calcAltitude(float pressure){

  float A = pressure/101325;
  float B = 1/5.25588;
  float C = pow(A,B);
  C = 1 - C;
  C = C /0.0000225577;

  return C;
}
